import Avatar from '@mui/material/Avatar'
import Button from '@mui/material/Button'
import TextField from '@mui/material/TextField'
import Link from '@mui/material/Link'
import Grid from '@mui/material/Unstable_Grid2'
import Box from '@mui/material/Box'
import AppRegistrationIcon from '@mui/icons-material/AppRegistration'
import Typography from '@mui/material/Typography'
import Container from '@mui/material/Container'
import { useFormik } from 'formik'
import { useRouter } from 'next/router'
import * as yup from '@/utils/validation'
import * as api from '@/api'
import { RegisterInputDto } from '@/api/user.interface'
import { EmailVerifyCodeScene } from '@/api/email.interface'
import { useCountdown, COUNTDOWN_SECONDS } from '@/utils/useCountdown'

export default function Register() {
  const router = useRouter()
  const { countdown, setCountdown } = useCountdown()

  const formik = useFormik<RegisterInputDto>({
    initialValues: {
      email: '',
      password: '',
      verifyCode: '',
      token: '',
    },
    validateOnChange: false,
    validationSchema: yup.object({
      email: yup.emailSchema,
      password: yup.passwordSchema,
      verifyCode: yup.verifyCodeSchema,
    }),
    onSubmit: async (values) => {
      const res = await api.user.register(values)
      localStorage.setItem('accessToken', res.data.accessToken)
      localStorage.setItem('refreshToken', res.data.refreshToken)
      router.push('/')
    },
  })

  async function sendVerifyCode() {
    await formik.validateField('email')
    if (formik.errors.email) return
    setCountdown(COUNTDOWN_SECONDS)
    try {
      const res = await api.email.sendEmailVerifyCode({
        email: formik.values.email,
        scene: EmailVerifyCodeScene.register,
      })
      formik.setFieldValue('token', res.data.token)
    } catch (err) {
      setCountdown(0)
    }
  }

  return (
    <Container component="main" sx={{ width: 400 }}>
      <Box
        sx={{
          marginTop: '20vh',
          display: 'flex',
          flexDirection: 'column',
          alignItems: 'center',
        }}
      >
        <Avatar sx={{ m: 1, bgcolor: 'primary.main' }}>
          <AppRegistrationIcon />
        </Avatar>
        <Typography component="h1" variant="h5">
          注册
        </Typography>
        <Box
          component="form"
          onSubmit={formik.handleSubmit}
          noValidate
          sx={{ mt: 1, width: '100%' }}
        >
          <TextField
            {...formik.getFieldProps('email')}
            required
            error={formik.touched.email && !!formik.errors.email}
            helperText={(formik.touched.email && formik.errors.email) || ' '}
            fullWidth
            label="邮箱"
            autoComplete="email"
            autoFocus
            sx={{ mt: 1 }}
          />
          <Grid container spacing={2}>
            <Grid>
              <TextField
                {...formik.getFieldProps('verifyCode')}
                required
                error={formik.touched.verifyCode && !!formik.errors.verifyCode}
                helperText={
                  (formik.touched.verifyCode && formik.errors.verifyCode) || ' '
                }
                fullWidth
                label="验证码"
                autoComplete="verifyCode"
                autoFocus
                sx={{ mt: 1 }}
              />
            </Grid>
            <Grid mt={2} sx={{ flexGrow: 1 }}>
              <Button
                size="large"
                fullWidth
                variant="contained"
                disabled={countdown > 0}
                onClick={sendVerifyCode}
              >
                {countdown > 0 ? countdown : '发送验证码'}
              </Button>
            </Grid>
          </Grid>
          <TextField
            {...formik.getFieldProps('password')}
            required
            error={formik.touched.password && !!formik.errors.password}
            helperText={
              (formik.touched.password && formik.errors.password) || ' '
            }
            fullWidth
            label="密码"
            type="password"
            autoComplete="current-password"
            sx={{ mt: 1 }}
          />
          <Button type="submit" fullWidth variant="contained" sx={{ mt: 1 }}>
            注册
          </Button>
          <Grid container justifyContent="flex-end" sx={{ mt: 2 }}>
            <Grid>
              <Typography component="span" variant="body2">
                已有账号？
              </Typography>
              <Link href="/login">登录</Link>
            </Grid>
          </Grid>
        </Box>
      </Box>
    </Container>
  )
}
